package com.jaywagnon.webservices.service.impl;


import com.jaywagnon.webservices.persistence.entity.User;
import com.jaywagnon.webservices.service.ModelNotFoundException;
import com.jaywagnon.webservices.service.UserService;
import com.jaywagnon.webservices.web.model.UserDTO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.MessageFormat;
import java.util.List;


@Service
public class UserRepositoryService
        implements UserService
{
  @Resource
  private com.jaywagnon.webservices.persistence.repository.UserRepository repository;


  @Transactional(readOnly = true)
  @Override
  public User find(final Long id)
          throws ModelNotFoundException
  {
    User entity = repository.findOne(id);
    if(entity == null)
      throw new ModelNotFoundException(
              MessageFormat.format("Unable to find a User with an id of {0}", id));
    return entity;
  }


  @Transactional(readOnly = true)
  @Override
  public List<User> findAll()
  {
    return repository.findAll();
  }


  @Transactional
  @Override
  public User create(final UserDTO model)
  {
    return repository.save(entityFromModel(model));
  }


  @Transactional(rollbackFor = ModelNotFoundException.class)
  @Override
  public User update(final Long id, final UserDTO model)
          throws ModelNotFoundException
  {
    return repository.save(entityFromModel(find(id), model));
  }


  @Transactional(rollbackFor = ModelNotFoundException.class)
  @Override
  public User delete(final Long id)
          throws ModelNotFoundException
  {
    User deleted = find(id);
    repository.delete(find(id));
    return deleted;
  }


  private User entityFromModel(UserDTO comment)
  {
    return entityFromModel(null, comment);
  }


  private User entityFromModel(User entity, UserDTO comment)
  {
    if(entity == null)
      entity = new User();

    if(comment != null) {
      entity.setFirstName(comment.getFirstName());
      entity.setLastName(comment.getLastName());
    }

    return entity;
  }
}
