package com.jaywagnon.webservices.web.controller;


import com.jaywagnon.webservices.service.ModelNotFoundException;
import com.jaywagnon.webservices.web.model.ValidationErrorDTO;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.http.HttpStatus;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;


@ControllerAdvice
public class RestErrorHandler
{
  private Log log = LogFactory.getLog(this.getClass());


  @ExceptionHandler(ModelNotFoundException.class)
  @ResponseStatus(HttpStatus.NOT_FOUND)
  public void handleModelNotFoundException(ModelNotFoundException exception)
  {
    log.debug(exception.getMessage());
  }


  @ExceptionHandler(MethodArgumentNotValidException.class)
  @ResponseStatus(HttpStatus.BAD_REQUEST)
  @ResponseBody
  public ValidationErrorDTO handleMethodArgumentNotValidException(MethodArgumentNotValidException exception)
  {
    ValidationErrorDTO dto = new ValidationErrorDTO();
    for(FieldError error : exception.getBindingResult().getFieldErrors()) {
      dto.add(error.getField(), error.getDefaultMessage());
    }
    return dto;
  }
}