package com.jaywagnon.webservices.web.controller;


import com.jaywagnon.webservices.persistence.entity.User;
import com.jaywagnon.webservices.service.ModelNotFoundException;
import com.jaywagnon.webservices.service.UserService;
import com.jaywagnon.webservices.web.model.UserDTO;
import com.jaywagnon.webservices.web.resources.UserResource;
import com.jaywagnon.webservices.web.resources.UserResourceAssembler;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.nio.charset.Charset;
import java.util.List;

import static org.springframework.hateoas.mvc.ControllerLinkBuilder.linkTo;
import static org.springframework.hateoas.mvc.ControllerLinkBuilder.methodOn;


@Controller
@RequestMapping(value = "/users")
public class UsersController
{
  @Resource
  private UserResourceAssembler UserResourceAssembler;


  public static final MediaType APPLICATION_JSON_UTF8 = new MediaType(
          MediaType.APPLICATION_JSON.getType(),
          MediaType.APPLICATION_JSON.getSubtype(),
          Charset.forName("utf8"));
  public static final String APPLICATION_JSON_UTF8_VALUE = "application/json;charset=UTF-8";


  @Autowired
  private UserService UserService;


  @RequestMapping(method = RequestMethod.GET, produces = APPLICATION_JSON_UTF8_VALUE)
  @ResponseBody
  public List<UserResource> getAll()
  {
    return UserResourceAssembler.toResources(UserService.findAll());
  }


  @RequestMapping(value = "/{id}", method = RequestMethod.GET, produces = APPLICATION_JSON_UTF8_VALUE)
  @ResponseBody
  public UserResource get(@PathVariable Long id)
          throws ModelNotFoundException
  {
    return UserResourceAssembler.toResource(UserService.find(id));
  }


  @RequestMapping(method = RequestMethod.POST, produces = APPLICATION_JSON_UTF8_VALUE)
  @ResponseBody
  public ResponseEntity<UserResource> create(@Valid @RequestBody UserDTO formObject)
          throws ModelNotFoundException
  {
    User user = UserService.create(formObject);
    HttpHeaders headers = new HttpHeaders();
    headers.setLocation(linkTo(methodOn(getClass()).get(user.getId())).toUri());
    return new ResponseEntity<>(
            UserResourceAssembler.toResource(user),
            headers,
            HttpStatus.CREATED);
  }


  @RequestMapping(value = "/{id}", method = RequestMethod.PUT, produces = APPLICATION_JSON_UTF8_VALUE)
  @ResponseBody
  public UserResource update(@PathVariable Long id, @Valid @RequestBody UserDTO formObject)
          throws ModelNotFoundException
  {
    return UserResourceAssembler.toResource(UserService.update(id, formObject));
  }


  @RequestMapping(value = "/{id}", method = RequestMethod.DELETE, produces = APPLICATION_JSON_UTF8_VALUE)
  @ResponseBody
  public User delete(@PathVariable Long id)
          throws ModelNotFoundException
  {
    return UserService.delete(id);
  }
}
