package com.jaywagnon.webservices.configuration;


import org.apache.commons.configuration.EnvironmentConfiguration;
import org.apache.commons.configuration.MapConfiguration;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;

import static com.jaywagnon.webservices.configuration.ConfigurationEnum.HIBERNATE_SHOW_SQL;
import static junit.framework.TestCase.*;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class ConfigurationEnumTest
{
  /**
   * Test for compiler injected Enum.values() for coverage
   */
  @Test
  public void test_values()
  {
    ConfigurationEnum[] values = ConfigurationEnum.values();
    assertEquals(7, values.length);
  }


  /**
   * Test for compiler injected Enum.valueOf() for coverage
   */
  @Test
  public void test_valueOf()
  {
    assertEquals(HIBERNATE_SHOW_SQL, ConfigurationEnum.valueOf("HIBERNATE_SHOW_SQL"));
  }


  @Test
  public void test_getConfiguration()
  {
    MapConfiguration mockConfiguration = mock(MapConfiguration.class);
    ConfigurationEnum.setConfiguration(mockConfiguration);

    Object configuration = ConfigurationEnum.getConfiguration();
    assertNotNull(configuration);
    assertEquals(mockConfiguration, configuration);
  }


  @Test
  public void test_getConfiguration_null()
  {
    ConfigurationEnum.setConfiguration(null);
    Object configuration = ConfigurationEnum.getConfiguration();
    assertNotNull(configuration);
    assertTrue(configuration instanceof EnvironmentConfiguration);
  }


  @Test
  public void test_defaultValue()
  {
    assertEquals("false", HIBERNATE_SHOW_SQL.defaultValue());
  }


  @Test
  public void test_value()
  {
    MapConfiguration configuration = mock(MapConfiguration.class);
    ConfigurationEnum.setConfiguration(configuration);

    when(configuration.getString(HIBERNATE_SHOW_SQL.toString())).thenReturn("Value");
    assertEquals("Value", HIBERNATE_SHOW_SQL.value());
    verify(configuration).getString(HIBERNATE_SHOW_SQL.toString());
  }


  @Test
  public void test_value_defaultValue()
  {
    MapConfiguration configuration = mock(MapConfiguration.class);
    ConfigurationEnum.setConfiguration(configuration);

    when(configuration.getString(HIBERNATE_SHOW_SQL.toString())).thenReturn(null);
    assertEquals(HIBERNATE_SHOW_SQL.defaultValue(), HIBERNATE_SHOW_SQL.value());
    verify(configuration).getString(HIBERNATE_SHOW_SQL.toString());
  }
}
