package com.jaywagnon.webservices.service.impl;


import com.jaywagnon.webservices.persistence.entity.User;
import com.jaywagnon.webservices.persistence.repository.UserRepository;
import com.jaywagnon.webservices.service.ModelNotFoundException;
import com.jaywagnon.webservices.web.model.UserDTO;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;

import java.util.Arrays;

import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertNotNull;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class UserRepositoryServiceTest
{
  private static final Long ID = 1L;

  @Mock
  private User entity;

  @Mock
  private UserDTO model;

  @Mock
  private UserRepository repository;

  @InjectMocks
  private UserRepositoryService service;


  @Before
  public void setupBeforeTest()
  {
    when(entity.getId()).thenReturn(ID);
    when(repository.findOne(entity.getId())).thenReturn(entity);
    when(repository.findAll()).thenReturn(Arrays.asList(entity));
  }


  @Test
  public void test_Constructor()
  {
    assertNotNull(service);
  }


  @Test
  public void test_find()
          throws ModelNotFoundException
  {
    service.find(ID);
    verify(repository, times(1)).findOne(ID);
  }


  @Test(expected = ModelNotFoundException.class)
  public void test_find_null()
          throws ModelNotFoundException
  {
    service.find(null);
    verify(repository, never()).findOne(ID);
  }


  @Test(expected = ModelNotFoundException.class)
  public void test_find_not_found()
          throws ModelNotFoundException
  {
    Long id = 100L;
    service.find(id);
    verify(repository, times(1)).findOne(id);
  }


  @Test
  public void test_findAll()
  {
    assertEquals(1, service.findAll().size());
    verify(repository, times(1)).findAll();
  }


  @Test
  public void test_create()
  {
    service.create(model);
    verify(repository).save(any(User.class));
  }


  @Test
  public void test_create_null()
  {
    service.create(null);
    verify(repository).save(any(User.class));
  }


  @Test
  public void test_update()
          throws ModelNotFoundException
  {
    service.update(1L, model);
    verify(repository).save(any(User.class));
  }


  @Test(expected = ModelNotFoundException.class)
  public void test_update_null()
          throws ModelNotFoundException
  {
    service.update(null, model);
    verify(repository, never()).save(any(User.class));
  }


  @Test
  public void test_delete()
          throws ModelNotFoundException
  {
    service.delete(ID);
    verify(repository).delete(entity);
  }


  @Test(expected = ModelNotFoundException.class)
  public void test_delete_by_id_null()
          throws ModelNotFoundException
  {
    service.delete(null);
    verify(repository, never()).delete(ID);
  }
}
