package com.jaywagnon.webservices.web.controller;


import com.jaywagnon.webservices.service.ModelNotFoundException;
import org.apache.commons.logging.Log;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;

import java.util.Arrays;

import static junit.framework.TestCase.assertNotNull;
import static org.mockito.Mockito.*;


@RunWith(MockitoJUnitRunner.class)
public class RestErrorHandlerTest
{
  @Mock
  private Log log;

  @InjectMocks
  private RestErrorHandler handler;


  @Test
  public void test_Constructor()
  {
    assertNotNull(handler);
  }


  @Test
  public void test_handleModelNotFoundException()
  {
    ModelNotFoundException exception = mock(ModelNotFoundException.class);
    handler.handleModelNotFoundException(exception);
    verify(exception, times(1)).getMessage();
    verify(log).debug(exception.getMessage());
  }


  @Test
  public void test_handleMethodArgumentNotValidException()
  {
    MethodArgumentNotValidException exception = mock(MethodArgumentNotValidException.class);
    BindingResult bindingResult = mock(BindingResult.class);
    when(exception.getBindingResult()).thenReturn(bindingResult);
    FieldError fieldError = mock(FieldError.class);
    when(bindingResult.getFieldErrors()).thenReturn(Arrays.asList(fieldError));

    handler.handleMethodArgumentNotValidException(exception);
    verify(exception, times(1)).getBindingResult();
    verify(bindingResult, times(1)).getFieldErrors();
    verify(fieldError, times(1)).getField();
    verify(fieldError, times(1)).getDefaultMessage();
  }
}
