package com.jaywagnon.webservices.web.resources;


import com.jaywagnon.webservices.persistence.entity.User;
import org.junit.Before;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.runners.MockitoJUnitRunner;
import org.springframework.hateoas.Link;
import org.springframework.mock.web.MockHttpServletRequest;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import java.util.Date;

import static junit.framework.TestCase.assertEquals;
import static junit.framework.TestCase.assertNotNull;


@RunWith(MockitoJUnitRunner.class)
public class UserResourceAssemblerTest
{
  private static String URL_BASE = "http://localhost/users";


  protected MockHttpServletRequest request;


  private User user = new User()
          .setId(1L)
          .setFirstName("First")
          .setLastName("Last")
          .setCreated(new Date())
          .setUpdated(new Date());


  private UserResourceAssembler assembler = new UserResourceAssembler();


  @Before
  public void setUp()
  {
    request = new MockHttpServletRequest();
    ServletRequestAttributes requestAttributes = new ServletRequestAttributes(request);
    RequestContextHolder.setRequestAttributes(requestAttributes);
  }


  @Test
  public void test_Constructor()
  {
    assertNotNull(assembler);
  }


  @Test
  public void test_toResource()
  {
    UserResource resource = assembler.toResource(user);
    assertEquals(user.getCreated(), resource.getCreated());
    assertEquals(user.getUpdated(), resource.getUpdated());
    assertEquals(user.getFirstName(), resource.getFirstName());
    assertEquals(user.getLastName(), resource.getLastName());

    Link link = resource.getId();
    assertEquals(URL_BASE + "/" + user.getId(), link.getHref());
  }


  @Test
  public void test_instantiateResource()
  {
    UserResource resource = assembler.toResource(user);
    assertEquals(user.getCreated(), resource.getCreated());
    assertEquals(user.getUpdated(), resource.getUpdated());
    assertEquals(user.getFirstName(), resource.getFirstName());
    assertEquals(user.getLastName(), resource.getLastName());

    Link link = resource.getId();
    assertEquals(URL_BASE + "/" + user.getId(), link.getHref());
  }
}
